@@ -479,9 +479,19 @@ impl Run {
         if let Some(file) = &task.file {
             self.exec_file(file, task, &env, &prefix)?;
         } else {
-            for (script, args) in
-                task.render_run_scripts_with_args(self.cd.clone(), &task.args, &env)?
-            {
+            let rendered_run_scripts =
+                task.render_run_scripts_with_args(self.cd.clone(), &task.args, &env)?;
+
+            let get_args = || {
+                [String::new()]
+                    .iter()
+                    .chain(task.args.iter())
+                    .cloned()
+                    .collect()
+            };
+            self.parse_usage_spec_and_init_env(task, &mut env, get_args)?;
+
+            for (script, args) in rendered_run_scripts {
                 self.exec_script(&script, &args, task, &env, &prefix)?;
             }
         }
@@ -596,14 +606,8 @@ impl Run {
         let mut env = env.clone();
         let command = file.to_string_lossy().to_string();
         let args = task.args.iter().cloned().collect_vec();
-        let (spec, _) = task.parse_usage_spec(self.cd.clone(), &env)?;
-        if !spec.cmd.args.is_empty() || !spec.cmd.flags.is_empty() {
-            let args = once(command.clone()).chain(args.clone()).collect_vec();
-            let po = usage::parse(&spec, &args).map_err(|err| eyre!(err))?;
-            for (k, v) in po.as_env() {
-                env.insert(k, v);
-            }
-        }
+        let get_args = || once(command.clone()).chain(args.clone()).collect_vec();
+        self.parse_usage_spec_and_init_env(task, &mut env, get_args)?;
 
         if !self.quiet(Some(task)) {
             let cmd = format!("{} {}", display_path(file), args.join(" "))
@@ -807,6 +811,28 @@ impl Run {
         Ok(())
     }
 
+    fn parse_usage_spec_and_init_env(
+        &self,
+        task: &Task,
+        env: &mut EnvMap,
+        get_args: impl Fn() -> Vec<String>,
+    ) -> Result<()> {
+        let (spec, _) = task.parse_usage_spec(self.cd.clone(), env)?;
+        if !spec.cmd.args.is_empty() || !spec.cmd.flags.is_empty() {
+            let args: Vec<String> = get_args();
+            debug!("Parsing usage spec for {:?}", args);
+            let po = usage::parse(&spec, &args).map_err(|err| eyre!(err))?;
+            for (k, v) in po.as_env() {
+                debug!("Adding key {} value {} in env", k, v);
+                env.insert(k, v);
+            }
+        } else {
+            debug!("Usage spec has no args or flags");
+        }
+
+        Ok(())
+    }
+
     fn sources_are_fresh(&self, task: &Task) -> Result<bool> {
         let outputs = task.outputs.paths(task);
         if task.sources.is_empty() && outputs.is_empty() {
