@@ -1,11 +1,11 @@
+use crate::config::Settings;
+use crate::dirs;
 use std::env::{join_paths, split_paths};
 use std::ffi::OsString;
 use std::fmt;
 use std::fmt::{Display, Formatter};
 use std::path::PathBuf;
-
-use crate::config::Settings;
-use crate::dirs;
+use std::str::FromStr;
 
 pub struct PathEnv {
     pre: Vec<PathBuf>,
@@ -74,6 +74,14 @@ impl FromIterator<PathBuf> for PathEnv {
     }
 }
 
+impl FromStr for PathEnv {
+    type Err = eyre::Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        Ok(Self::from_iter(split_paths(s)))
+    }
+}
+
 #[cfg(unix)]
 #[cfg(test)]
 mod tests {
